import random
import datetime as dt

from pycalver import version


def test_current_calver():
    v = version.current_calver()
    assert len(v) == 7
    assert v.startswith("v")
    assert v[1:].isdigit()


def test_bump_beta():
    calver      = version.current_calver()
    cur_version = calver + ".0001-beta"
    assert cur_version < version.incr(cur_version)
    assert version.incr(cur_version).endswith("-beta")
    assert version.incr(cur_version, release="alpha").endswith("-alpha")
    assert version.incr(cur_version, release="final").endswith("0002")


def test_bump_final():
    calver      = version.current_calver()
    cur_version = calver + ".0001"
    assert cur_version < version.incr(cur_version)
    assert version.incr(cur_version, release="alpha").endswith("-alpha")
    assert version.incr(cur_version, release="final").endswith("0002")
    assert version.incr(cur_version).endswith("0002")


def test_bump_future():
    future_date   = dt.datetime.today() + dt.timedelta(days=300)
    future_calver = future_date.strftime("v%Y%m")
    cur_version   = future_calver + ".0001"
    assert cur_version < version.incr(cur_version)


def test_bump_random(monkeypatch):
    cur_date    = dt.date.today()
    cur_version = cur_date.strftime("v%Y%m") + ".0001-dev"

    def _mock_current_calver():
        return cur_date.strftime("v%Y%m")

    monkeypatch.setattr(version, 'current_calver', _mock_current_calver)

    for i in range(1000):
        cur_date += dt.timedelta(days=int((1 + random.random()) ** 10))
        new_version = version.incr(
            cur_version, release=random.choice([None, "alpha", "beta", "rc", 'final'])
        )
        assert cur_version < new_version
        cur_version = new_version


def test_parse_version_info():
    version_str = "v201712.0001-alpha"
    version_nfo = version.parse_version_info(version_str)

    assert version_nfo.pep440_version == "201712.1a0"
    assert version_nfo.version        == "v201712.0001-alpha"
    assert version_nfo.calver         == "v201712"
    assert version_nfo.year           == "2017"
    assert version_nfo.month          == "12"
    assert version_nfo.build          == ".0001"
    assert version_nfo.release        == "-alpha"

    version_str = "v201712.0001"
    version_nfo = version.parse_version_info(version_str)

    assert version_nfo.pep440_version == "201712.1"
    assert version_nfo.version        == "v201712.0001"
    assert version_nfo.calver         == "v201712"
    assert version_nfo.year           == "2017"
    assert version_nfo.month          == "12"
    assert version_nfo.build          == ".0001"
    assert version_nfo.release is None


def test_readme_pycalver1():
    version_str  = "v201712.0001-alpha"
    version_info = version.PYCALVER_RE.match(version_str).groupdict()

    assert version_info == {
        'version': "v201712.0001-alpha",
        'calver' : "v201712",
        'year'   : "2017",
        'month'  : "12",
        'build'  : ".0001",
        'release': "-alpha",
    }


def test_readme_pycalver2():
    version_str  = "v201712.0033"
    version_info = version.PYCALVER_RE.match(version_str).groupdict()

    assert version_info == {
        'version': "v201712.0033",
        'calver' : "v201712",
        'year'   : "2017",
        'month'  : "12",
        'build'  : ".0033",
        'release': None,
    }


def test_parse_error_empty():
    try:
        version.parse_version_info("")
        assert False
    except ValueError as err:
        pass


def test_parse_error_noprefix():
    try:
        version.parse_version_info("201809.0002")
        assert False
    except ValueError as err:
        pass


def test_parse_error_nopadding():
    try:
        version.parse_version_info("v201809.2b0")
        assert False
    except ValueError as err:
        pass



# This file is part of the pycalver project
# https://github.com/mbarkhau/pycalver
#
# Copyright (c) 2018 Manuel Barkhau (@mbarkhau) - MIT License
# SPDX-License-Identifier: MIT
"""Functions related to version string manipulation.

>>> version_info = PYCALVER_RE.match("v201712.0123-alpha").groupdict()
>>> assert version_info == {
...     "version" : "v201712.0123-alpha",
...     "calver"  : "v201712",
...     "year"    : "2017",
...     "month"   : "12",
...     "build"   : ".0123",
...     "release" : "-alpha",
... }
>>>
>>> version_info = PYCALVER_RE.match("v201712.0033").groupdict()
>>> assert version_info == {
...     "version" : "v201712.0033",
...     "calver"  : "v201712",
...     "year"    : "2017",
...     "month"   : "12",
...     "build"   : ".0033",
...     "release" : None,
... }
"""

import re
import logging
import pkg_resources
import typing as typ
import datetime as dt

from . import lex_id

log = logging.getLogger("pycalver.version")


# https://regex101.com/r/fnj60p/10
PYCALVER_PATTERN = r"""
\b
(?P<version>
    (?P<calver>
       v                        # "v" version prefix
       (?P<year>\d{4})
       (?P<month>\d{2})
    )
    (?P<build>
        \.                      # "." build nr prefix
        \d{4,}
    )
    (?P<release>
        \-                      # "-" release prefix
        (?:alpha|beta|dev|rc|post)
    )?
)(?:\s|$)
"""

PYCALVER_RE: typ.Pattern[str] = re.compile(PYCALVER_PATTERN, flags=re.VERBOSE)


class VersionInfo(typ.NamedTuple):
    """Container for parsed version string."""

    version       : str
    pep440_version: str
    calver        : str
    year          : str
    month         : str
    build         : str
    release       : typ.Optional[str]


def parse_version_info(version_str: str) -> VersionInfo:
    """Parse a PyCalVer string.

    >>> vnfo = parse_version_info("v201712.0033-beta")
    >>> assert vnfo == VersionInfo(
    ...     version="v201712.0033-beta",
    ...     pep440_version="201712.33b0",
    ...     calver="v201712",
    ...     year="2017",
    ...     month="12",
    ...     build=".0033",
    ...     release="-beta",
    ... )
    """
    match = PYCALVER_RE.match(version_str)
    if match is None:
        raise ValueError(f"Invalid PyCalVer string: {version_str}")

    kwargs = match.groupdict()
    kwargs['pep440_version'] = pycalver_to_pep440(kwargs['version'])
    return VersionInfo(**kwargs)


def current_calver() -> str:
    """Generate calver version string based on current date.

    example result: "v201812"
    """
    return dt.date.today().strftime("v%Y%m")


def incr(old_version: str, *, release: str = None) -> str:
    """Increment a full PyCalVer version string.

    Old_version is assumed to be a valid calver string,
    already validated in pycalver.config.parse.
    """
    old_ver = parse_version_info(old_version)

    new_calver = current_calver()

    if old_ver.calver > new_calver:
        log.warning(
            f"'version.incr' called with '{old_version}', "
            + f"which is from the future, "
            + f"maybe your system clock is out of sync."
        )
        # leave calver as is (don't go back in time)
        new_calver = old_ver.calver

    new_build = lex_id.next_id(old_ver.build[1:])
    new_release: typ.Optional[str] = None

    if release is None:
        if old_ver.release:
            # preserve existing release
            new_release = old_ver.release[1:]
        else:
            new_release = None
    elif release == 'final':
        new_release = None
    else:
        new_release = release

    new_version = new_calver + "." + new_build
    if new_release:
        new_version += "-" + new_release
    return new_version


def pycalver_to_pep440(version: str) -> str:
    """Derive pep440 compliant version string from PyCalVer version string.

    >>> pycalver_to_pep440("v201811.0007-beta")
    '201811.7b0'
    """
    return str(pkg_resources.parse_version(version))



# This file is part of the pycalver project
# https://github.com/mbarkhau/pycalver
#
# Copyright (c) 2018 Manuel Barkhau (@mbarkhau) - MIT License
# SPDX-License-Identifier: MIT
"""Parse PyCalVer strings from files."""

import re
import logging
import typing as typ

log = logging.getLogger("pycalver.parse")


VALID_RELEASE_VALUES = ("alpha", "beta", "dev", "rc", "post")


PATTERN_ESCAPES = [
    ("\u005c", "\u005c\u005c"),
    ("-"     , "\u005c-"),
    ("."     , "\u005c."),
    ("+"     , "\u005c+"),
    ("*"     , "\u005c*"),
    ("["     , "\u005c["),
    ("("     , "\u005c("),
]

# NOTE (mb 2018-09-03): These are matchers for parts, which are
#   used in the patterns, they're not for validation. This means
#   that they may find strings, which are not valid pycalver
#   strings, when parsed in their full context. For such cases,
#   the patterns should be expanded.


RE_PATTERN_PARTS = {
    'pep440_version': r"\d{6}\.[1-9]\d*(a|b|dev|rc|post)?\d*",
    'version'       : r"v\d{6}\.\d{4,}(\-(alpha|beta|dev|rc|post))?",
    'calver'        : r"v\d{6}",
    'build'         : r"\.\d{4,}",
    'release'       : r"(\-(alpha|beta|dev|rc|post))?",
}


class PatternMatch(typ.NamedTuple):
    """Container to mark a version string in a file."""

    lineno : int  # zero based
    line   : str
    pattern: str
    span   : typ.Tuple[int, int]
    match  : str


PatternMatches = typ.Iterable[PatternMatch]


def _iter_for_pattern(lines: typ.List[str], pattern: str) -> PatternMatches:
    # The pattern is escaped, so that everything besides the format
    # string variables is treated literally.

    pattern_tmpl = pattern

    for char, escaped in PATTERN_ESCAPES:
        pattern_tmpl = pattern_tmpl.replace(char, escaped)

    pattern_str = pattern_tmpl.format(**RE_PATTERN_PARTS)
    pattern_re  = re.compile(pattern_str)
    for lineno, line in enumerate(lines):
        match = pattern_re.search(line)
        if match:
            yield PatternMatch(lineno, line, pattern, match.span(), match.group(0))


def iter_matches(lines: typ.List[str], patterns: typ.List[str]) -> PatternMatches:
    """Iterate over all matches of any pattern on any line.

    >>> lines = ["__version__ = 'v201712.0002-alpha'"]
    >>> patterns = ["{version}", "{pep440_version}"]
    >>> matches = list(iter_matches(lines, patterns))
    >>> assert matches[0] == PatternMatch(
    ...     lineno = 0,
    ...     line   = "__version__ = 'v201712.0002-alpha'",
    ...     pattern= "{version}",
    ...     span   = (15, 33),
    ...     match  = "v201712.0002-alpha",
    ... )
    """
    for pattern in patterns:
        for match in _iter_for_pattern(lines, pattern):
            yield match



import re
from pycalver import parse


def test_re_pattern_parts():
    part_re_by_name = {
        part_name: re.compile(part_re_str)
        for part_name, part_re_str in parse.RE_PATTERN_PARTS.items()
    }

    cases = [
        ("pep440_version", "201712.31"          , "201712.31"),
        ("pep440_version", "v201712.0032"       , None),
        ("pep440_version", "201712.0033-alpha"  , None),
        ("version"       , "v201712.0034"       , "v201712.0034"),
        ("version"       , "v201712.0035-alpha" , "v201712.0035-alpha"),
        ("version"       , "v201712.0036-alpha0", "v201712.0036-alpha"),
        ("version"       , "v201712.0037-pre"   , "v201712.0037"),
        ("version"       , "201712.38a0"        , None),
        ("version"       , "201712.0039"        , None),
        ("calver"        , "v201712"            , "v201712"),
        ("calver"        , "v201799"            , "v201799"),  # maybe date validation should be a thing
        ("calver"        , "201712"             , None),
        ("calver"        , "v20171"             , None),
        ("build"         , ".0012"              , ".0012"),
        ("build"         , ".11012"             , ".11012"),
        ("build"         , ".012"               , None),
        ("build"         , "11012"              , None),
        ("release"       , "-alpha"             , "-alpha"),
        ("release"       , "-beta"              , "-beta"),
        ("release"       , "-dev"               , "-dev"),
        ("release"       , "-rc"                , "-rc"),
        ("release"       , "-post"              , "-post"),
        ("release"       , "-pre"               , ""),
        ("release"       , "alpha"              , ""),
    ]

    for part_name, line, expected in cases:
        part_re = part_re_by_name[part_name]
        result  = part_re.search(line)
        if result is None:
            assert expected is None, (part_name, line)
        else:
            result_val = result.group(0)
            assert result_val == expected, (part_name, line)


SETUP_PY_FIXTURE = """
# setup.py
import setuptools
__version__ = 'v201712.0002-alpha'
setuptools.setup(
...
   version='201712.2a0',
"""


def test_default_parse_patterns():
    lines    = SETUP_PY_FIXTURE.splitlines()
    patterns = ["{version}", "{pep440_version}"]

    matches = list(parse.iter_matches(lines, patterns))
    assert len(matches) == 2

    assert matches[0].lineno == 3
    assert matches[1].lineno == 6

    assert matches[0].pattern == patterns[0]
    assert matches[1].pattern == patterns[1]

    assert matches[0].match == "v201712.0002-alpha"
    assert matches[1].match == "201712.2a0"


def test_explicit_parse_patterns():
    lines = SETUP_PY_FIXTURE.splitlines()

    patterns = ["__version__ = '{version}'", "version='{pep440_version}'"]

    matches = list(parse.iter_matches(lines, patterns))
    assert len(matches) == 2

    assert matches[0].lineno == 3
    assert matches[1].lineno == 6

    assert matches[0].pattern == patterns[0]
    assert matches[1].pattern == patterns[1]

    assert matches[0].match == "__version__ = 'v201712.0002-alpha'"
    assert matches[1].match == "version='201712.2a0'"


README_RST_FIXTURE = """
:alt: PyPI version

.. |version| image:: https://img.shields.io/badge/CalVer-v201809.0002--beta-blue.svg
:target: https://calver.org/
:alt: CalVer v201809.0002-beta
"""


def test_badge_parse_patterns():
    lines = README_RST_FIXTURE.splitlines()

    patterns = ["badge/CalVer-{calver}{build}-{release}-blue.svg", ":alt: CalVer {version}"]

    matches = list(parse.iter_matches(lines, patterns))
    assert len(matches) == 2

    assert matches[0].lineno == 3
    assert matches[1].lineno == 5

    assert matches[0].pattern == patterns[0]
    assert matches[1].pattern == patterns[1]

    assert matches[0].match == "badge/CalVer-v201809.0002--beta-blue.svg"
    assert matches[1].match == ":alt: CalVer v201809.0002-beta"



# This file is part of the pycalver project
# https://github.com/mbarkhau/pycalver
#
# (C) 2018 Manuel Barkhau (@mbarkhau)
# SPDX-License-Identifier: MIT
"""Rewrite files, updating occurences of version strings."""

import io
import difflib
import logging
import typing as typ

from . import parse
from . import config
from . import version


log = logging.getLogger("pycalver.rewrite")


def detect_line_sep(content: str) -> str:
    r"""Parse line separator from content.

    >>> detect_line_sep('\r\n')
    '\r\n'
    >>> detect_line_sep('\r')
    '\r'
    >>> detect_line_sep('\n')
    '\n'
    >>> detect_line_sep('')
    '\n'
    """
    if "\r\n" in content:
        return "\r\n"
    elif "\r" in content:
        return "\r"
    else:
        return "\n"


def rewrite_lines(
    patterns: typ.List[str], new_version: str, old_lines: typ.List[str]
) -> typ.List[str]:
    """Replace occurances of patterns in old_lines with new_version.

    >>> old_lines = ['__version__ = "v201809.0002-beta"']
    >>> patterns = ['__version__ = "{version}"']
    >>> new_lines = rewrite_lines(patterns, "v201811.0123-beta", old_lines)
    >>> assert new_lines == ['__version__ = "v201811.0123-beta"']
    """
    new_version_nfo        = version.parse_version_info(new_version)
    new_version_fmt_kwargs = new_version_nfo._asdict()

    new_lines = old_lines[:]

    for m in parse.iter_matches(old_lines, patterns):
        replacement = m.pattern.format(**new_version_fmt_kwargs)
        span_l, span_r = m.span
        new_line = m.line[:span_l] + replacement + m.line[span_r:]
        new_lines[m.lineno] = new_line

    return new_lines


class RewrittenFileData(typ.NamedTuple):
    """Container for line-wise content of rewritten files."""

    path     : str
    line_sep : str
    old_lines: typ.List[str]
    new_lines: typ.List[str]


def rfd_from_content(patterns: typ.List[str], new_version: str, content: str) -> RewrittenFileData:
    r"""Rewrite pattern occurrences with version string.

    >>> patterns = ['__version__ = "{version}"']
    >>> content = '__version__ = "v201809.0001-alpha"'
    >>> rfd = rfd_from_content(patterns, "v201809.0123", content)
    >>> assert rfd.new_lines == ['__version__ = "v201809.0123"']
    """
    line_sep  = detect_line_sep(content)
    old_lines = content.split(line_sep)
    new_lines = rewrite_lines(patterns, new_version, old_lines)
    return RewrittenFileData("<path>", line_sep, old_lines, new_lines)


def iter_rewritten(
    file_patterns: config.PatternsByFilePath, new_version: str
) -> typ.Iterable[RewrittenFileData]:
    r'''Iterate over files with version string replaced.

    >>> file_patterns = {"src/pycalver/__init__.py": ['__version__ = "{version}"']}
    >>> rewritten_datas = iter_rewritten(file_patterns, "v201809.0123")
    >>> rfd = list(rewritten_datas)[0]
    >>> assert rfd.new_lines == [
    ...     '# This file is part of the pycalver project',
    ...     '# https://gitlab.com/mbarkhau/pycalver',
    ...     '#',
    ...     '# Copyright (c) 2018 Manuel Barkhau (mbarkhau@gmail.com) - MIT License',
    ...     '# SPDX-License-Identifier: MIT',
    ...     '"""PyCalVer: Automatic CalVer Versioning for Python Packages."""',
    ...     '',
    ...     '__version__ = "v201809.0123"',
    ...     '',
    ... ]
    >>>
    '''
    for filepath, patterns in file_patterns.items():
        with io.open(filepath, mode="rt", encoding="utf-8") as fh:
            content = fh.read()

        rfd = rfd_from_content(patterns, new_version, content)
        yield rfd._replace(path=filepath)


def diff_lines(rfd: RewrittenFileData) -> typ.List[str]:
    r"""Generate unified diff.

    >>> rfd = RewrittenFileData(
    ...    path      = "<path>",
    ...    line_sep  = "\n",
    ...    old_lines = ["foo"],
    ...    new_lines = ["bar"],
    ... )
    >>> diff_lines(rfd)
    ['--- <path>', '+++ <path>', '@@ -1 +1 @@', '-foo', '+bar']
    """
    lines = difflib.unified_diff(
        a=rfd.old_lines, b=rfd.new_lines, lineterm="", fromfile=rfd.path, tofile=rfd.path
    )
    return list(lines)


def diff(new_version: str, file_patterns: config.PatternsByFilePath) -> str:
    r"""Generate diffs of rewritten files.

    >>> file_patterns = {"src/pycalver/__init__.py": ['__version__ = "{version}"']}
    >>> diff_str = diff("v201809.0123", file_patterns)
    >>> lines = diff_str.split("\n")
    >>> lines[:2]
    ['--- src/pycalver/__init__.py', '+++ src/pycalver/__init__.py']
    >>> assert lines[6].startswith('-__version__ = "v2')
    >>> assert not lines[6].startswith('-__version__ = "v201809.0123"')
    >>> lines[7]
    '+__version__ = "v201809.0123"'
    """

    full_diff = ""
    file_path: str
    for file_path, patterns in file_patterns.items():
        with io.open(file_path, mode="rt", encoding="utf-8") as fh:
            content = fh.read()

        rfd = rfd_from_content(patterns, new_version, content)
        rfd = rfd._replace(path=file_path)
        full_diff += "\n".join(diff_lines(rfd)) + "\n"

    full_diff = full_diff.rstrip("\n")
    return full_diff


def rewrite(new_version: str, file_patterns: config.PatternsByFilePath) -> None:
    """Rewrite project files, updating each with the new version."""

    for file_data in iter_rewritten(file_patterns, new_version):
        new_content = file_data.line_sep.join(file_data.new_lines)
        with io.open(file_data.path, mode="wt", encoding="utf-8") as fh:
            fh.write(new_content)



# This file is part of the pycalver project
# https://github.com/mbarkhau/pycalver
#
# Copyright (c) 2018 Manuel Barkhau (@mbarkhau) - MIT License
# SPDX-License-Identifier: MIT
#
# pycalver/vcs.py (this file) is based on code from the
# bumpversion project: https://github.com/peritus/bumpversion
# Copyright (c) 2013-2014 Filip Noetzel - MIT License
"""Minimal Git and Mercirial API.

If terminology for similar concepts differs between git and
mercurial, then the git terms are used. For example "fetch"
(git) instead of "pull" (hg) .
"""

import os
import logging
import tempfile
import typing as typ
import subprocess as sp


log = logging.getLogger("pycalver.vcs")


VCS_SUBCOMMANDS_BY_NAME = {
    'git': {
        'is_usable': "git rev-parse --git-dir",
        'fetch'    : "git fetch",
        'ls_tags'  : "git tag --list v*",
        'status'   : "git status --porcelain",
        'add_path' : "git add --update {path}",
        'commit'   : "git commit --file {path}",
        'tag'      : "git tag --annotate {tag} --message {tag}",
        'push_tag' : "git push origin {tag}",
    },
    'hg': {
        'is_usable': "hg root",
        'fetch'    : "hg pull",
        'ls_tags'  : "hg tags",
        'status'   : "hg status -mard",
        'add_path' : "hg add {path}",
        'commit'   : "hg commit --logfile",
        'tag'      : "hg tag {tag} --message {tag}",
        'push_tag' : "hg push {tag}",
    },
}


class VCS:
    """VCS absraction for git and mercurial."""

    def __init__(self, name: str, subcommands: typ.Dict[str, str] = None):
        self.name = name
        if subcommands is None:
            self.subcommands = VCS_SUBCOMMANDS_BY_NAME[name]
        else:
            self.subcommands = subcommands

    def __call__(self, cmd_name: str, env=None, **kwargs: str) -> str:
        """Invoke subcommand and return output."""
        cmd_str     = self.subcommands[cmd_name]
        cmd_parts   = cmd_str.format(**kwargs).split()
        output_data = sp.check_output(cmd_parts, env=env)

        # TODO (mb 2018-11-15): Detect encoding of output?
        _encoding = "utf-8"
        return output_data.decode(_encoding)

    @property
    def is_usable(self) -> bool:
        """Detect availability of subcommand."""
        cmd = self.subcommands['is_usable'].split()

        try:
            retcode = sp.call(cmd, stderr=sp.PIPE, stdout=sp.PIPE)
            return retcode == 0
        except OSError as e:
            if e.errno == 2:
                # git/mercurial is not installed.
                return False
            raise

    def fetch(self) -> None:
        """Fetch updates from remote origin."""
        self('fetch')

    def status(self) -> typ.List[str]:
        """Get status lines."""
        status_output = self('status')
        return [
            line[2:].strip()
            for line in status_output.splitlines()
            if not line.strip().startswith("??")
        ]

    def ls_tags(self) -> typ.List[str]:
        """List vcs tags on all branches."""
        ls_tag_lines = self('ls_tags').splitlines()
        log.debug(f"ls_tags output {ls_tag_lines}")
        return [line.strip() for line in ls_tag_lines if line.strip().startswith("v")]

    def add(self, path: str) -> None:
        """Add updates to be included in next commit."""
        log.info(f"{self.name} add {path}")
        self('add_path', path=path)

    def commit(self, message: str) -> None:
        """Commit added files."""
        log.info(f"{self.name} commit -m '{message}'")
        message_data = message.encode("utf-8")

        tmp_file = tempfile.NamedTemporaryFile("wb", delete=False)
        assert " " not in tmp_file.name

        with tmp_file as fh:
            fh.write(message_data)

        env = os.environ.copy()
        # TODO (mb 2018-09-04): check that this works on py27,
        #   might need to be bytes there, idk.
        env['HGENCODING'] = "utf-8"
        self('commit', env=env, path=tmp_file.name)
        os.unlink(tmp_file.name)

    def tag(self, tag_name: str) -> None:
        """Create an annotated tag."""
        self('tag', tag=tag_name)

    def push(self, tag_name: str) -> None:
        """Push changes to origin."""
        self('push_tag', tag=tag_name)

    def __repr__(self) -> str:
        """Generate string representation."""
        return f"VCS(name='{self.name}')"


def get_vcs() -> VCS:
    """Detect the appropriate VCS for a repository.

    raises OSError if the directory doesn't use a supported VCS.
    """
    for vcs_name in VCS_SUBCOMMANDS_BY_NAME.keys():
        vcs = VCS(name=vcs_name)
        if vcs.is_usable:
            return vcs

    raise OSError("No such directory .git/ or .hg/ ")



from pycalver import rewrite


REWRITE_FIXTURE = """
# This file is part of the pycalver project
# https://github.com/mbarkhau/pycalver
#
# (C) 2018 Manuel Barkhau (@mbarkhau)
# SPDX-License-Identifier: MIT

__version__ = "v201809.0002-beta"
"""


def test_rewrite_lines():
    old_lines   = REWRITE_FIXTURE.splitlines()
    patterns    = ['__version__ = "{version}"']
    new_version = "v201809.0003"
    new_lines   = rewrite.rewrite_lines(patterns, new_version, old_lines)

    assert len(new_lines) == len(old_lines)
    assert new_version not in "\n".join(old_lines)
    assert new_version     in "\n".join(new_lines)
